# -*- coding: utf-8 -*-

import pytest
import ast


def pytest_addoption(parser):
    group = parser.getgroup('general')
    group.addoption(
        '--cipdb',
        action='store_true',
        dest='check_ipdb',
        help='perform ipdb sanity check on .py files'
    )


def pytest_collect_file(parent, path):
    config = parent.config
    if path.ext != '.py' or not config.option.check_ipdb:
        return
    return CheckIpdbItem(path, parent=parent)


class CheckIpdbItem(pytest.Item, pytest.File):

    def __init__(self, path, parent):
        super(CheckIpdbItem, self).__init__(path, parent=parent)
        self.raw_content = self.fspath.open().read()

    def runtest(self):
        t = ast.parse(self.raw_content)
        Visitor().visit(t)

    def repr_failure(self, excinfo):
        if excinfo.errisinstance(CheckIpdbError):
            return '{} in file {}'.format(excinfo.value.args[0], self.fspath)
        return super(CheckIpdbItem, self).repr_failure(excinfo)

    def reportinfo(self):
        return self.fspath, -1, "cipdb-check"


class CheckIpdbError(Exception):
    """ indicates an error during cipdb checks. """


class Visitor(ast.NodeVisitor):

    check_functions = ['ipdb', 'pdb', 'set_trace', 'breakpoint']

    def visit_Call(self, node):
        try:
            object_to_evaluate = node.func
            try:
                attribute_to_evaluate = node.func.id
            except AttributeError:
                attribute_to_evaluate = node.func.attr
            for item in self.check_functions:
                if attribute_to_evaluate == item:
                    line_number = object_to_evaluate.lineno
                    col_number = object_to_evaluate.col_offset
                    raise CheckIpdbError('Detected {} call at line {} col {}'.format(item, line_number, col_number))
        except AttributeError:
            pass
        ast.NodeVisitor.generic_visit(self, node)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import codecs
from setuptools import setup


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding='utf-8').read()


setup(
    name='pytest-checkipdb',
    version='1.0.6',
    author='Andreu Vallbona',
    author_email='avallbona@gmail.com',
    maintainer='Andreu Vallbona',
    maintainer_email='avallbona@gmail.com',
    license='MIT',
    url='https://github.com/avallbona/pytest-checkipdb',
    description='plugin to check if there are ipdb debugs left',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    py_modules=[
        'pytest_checkipdb'
    ],
    install_requires=[
        'pytest>=2.9.2',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Pytest',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
    ],
    entry_points={
        'pytest11': [
            'checkipdb = pytest_checkipdb',
        ],
    },
)
